// Webservice Device-Management 
// AGENT
// Author: Oliver Bohle o.bohle@tu-bs.de
// Technische Universitt Braunschweig
#include "agent.h"
#include "../headers/SOAPdevmgrStub.h" 
#include "../headers/myns.nsmap"
#include <signal.h>

char outfile[60];
struct nsif__iflist *ifl;
xsd__int int_result;
double res;
struct soap soap;


/* structures to hold temporary information for the sending process */
struct nsif__interfaceDescription *tmpifDescr;
int tmpifDescr_size;
struct nsrt__routingInformation *tmprtInf;
int tmprtInf_size;
struct nsif__interfaceStats *tmpifStat;
int tmpifStat_size;





/* function to call the updatefunctions for the global route and
   interface information arrays */
void updateServiceInformation(){
#ifdef AGENT_DEBUG
  printf("UPDATE SERVICEINFORAMTION\n");
#endif
  update_InterfaceList();
  update_RoutingInformation();
}

/* copies an interfaceDescription struct from Parameter from to
   Parameter to. The to struct must not be a NULL-pionter */
void cpifDescr(struct nsif__interfaceDescription *to,struct nsif__interfaceDescription *from){
  int i;
  to->name=calloc(strlen(from->name)+1,sizeof(char));
  strcpy(to->name,from->name);
  to->type=from->type;
  
  to->mtu=from->mtu;

  /* Addresses */
 
  to->address=cp_ns__addr(to->address,from->address);
  to->netmask=cp_ns__addr(to->netmask,from->netmask);
  to->broadcast=cp_ns__addr(to->broadcast,from->broadcast);
  to->dst_address=cp_ns__addr(to->dst_address,from->dst_address);
  to->link_address=cp_ns__addr(to->link_address,from->link_address);
  
  
  /* Flags */
  to->__sizeflags=from->__sizeflags;
  if (from->__sizeflags>0){
    int flags=0;
    struct flagContainer rfc;
    flags=setInterfaceFlags(from);
    getInterfaceFlags(flags,&rfc);
    to->flag=rfc.array;
  }else{
    to->flag=NULL;
  }

  /* ALIAS */
  to->__sizealias=from->__sizealias;
  to->alias=(struct ns__address **) calloc(to->__sizealias,sizeof(struct ns__address *));

  for (i = 0; i < to->__sizealias; i++){
    to->alias[i]=cp_ns__addr(to->alias[i],from->alias[i]);
  }
  	     
  
  

  
} 


/* function called by soap_serve on a incoming uptime-Request */
int nssys__uptime(struct soap *soap,xsd__string *result){
  struct ut *uptime;
  char *out=soap_malloc(soap,60*sizeof(char));

  uptime=readProc_uptime();
  sprintf(out,"Uptime %d days %d hours %d minutes %d seconds",uptime->uptime->day,uptime->uptime->hour,uptime->uptime->min,uptime->uptime->sec);
  *result=out;
  free(uptime->uptime);
  free(uptime->idle);
  free(uptime);
  
  return SOAP_OK;
}


/* function called by soap_serve on a incoming
   getSystemInformation-Request */
int nssys__getSystemInformation(struct soap *soap,struct nssys__sysInf *result){

  char *tmp;
  struct ut *upt;

  tmp=getSystemDescription();
  result->description=(char *) soap_malloc(soap,strlen(tmp)*sizeof(char));
  strcpy(result->description,tmp); 
  free(tmp);
  upt=readProc_uptime();
  tmp=getXSDDuration(upt->uptime);
  result->uptime=(char *) soap_malloc(soap,strlen(tmp)*sizeof(char));
  strcpy(result->uptime,tmp);
  free(tmp);
  tmp=getXSDDuration(upt->idle);
  result->idletime=(char *) soap_malloc(soap,strlen(tmp)*sizeof(char));
  strcpy(result->idletime,tmp);
  free(tmp);
  free(upt->uptime);
  free(upt->idle);
  free(upt);
  return SOAP_OK;
}

/* frees the with parameter id given interfaceDescription-array. The
   number of elements of this array will be determined by the
   parameter size.  */
struct  nsif__interfaceDescription * freeInterfaceDescriptionArray(struct nsif__interfaceDescription *id,int *size){
  int i=0;
  if (id!=NULL){
    for(i=0;i<*size;i++)
      freeInterfaceDescription(&id[i]);
    id=NULL;
    size=0;
  }
  size=0;
  return id;
}


/* function called by soap_serve on a incoming
   getInterfaceInformation-Request */
int nsif__getInterfaceInformation(struct soap *soap, struct nsif__interfaceDescription in,struct interfacelist *result){
  int i=0;
  tmpifDescr=freeInterfaceDescriptionArray(tmpifDescr,&tmpifDescr_size);
  tmpifDescr=NULL;
  tmpifDescr_size=0;
#ifdef AGNENT_DEBUG
  print_interfaceDescription(&in);
#endif
  
  if ((in.name==NULL) || (strcmp(in.name,"")==0)) {
    /* all interfaceinformation are requested */
    result->__size=ifDescr_size;
    result->__offset=0;
    result->__ptr=ifDescr;
    return SOAP_OK;
  }else{
    for(i=0;i<ifDescr_size;i++){
      if (strcmp(ifDescr[i].name,in.name)==0){
	++tmpifDescr_size;
	tmpifDescr=(struct nsif__interfaceDescription *) realloc(tmpifDescr,tmpifDescr_size*sizeof(struct nsif__interfaceDescription));
	bzero(&tmpifDescr[tmpifDescr_size-1],sizeof(struct nsif__interfaceDescription));
	if (tmpifDescr==NULL) error("nsif__getInterfaceList","Not enough Memory",-2);
	cpifDescr(&tmpifDescr[tmpifDescr_size-1],&ifDescr[i]);
      }
    }   
    result->__size=tmpifDescr_size;
    result->__offset=0;
    result->__ptr=tmpifDescr;
    return SOAP_OK;
  }
  return SOAP_FAULT;
}

/* function called by soap_serve on a incoming
   setInterfaceInformation-Request */
int nsif__setInterfaceInformation(struct soap *soap, struct nsif__interfaceDescription in,struct interfacelist *result){
int i=0;
  char *errmsg=(char *)soap_malloc(rm_soap,AGENT_ERRMSGLEN*sizeof(char));
  tmpifDescr=freeInterfaceDescriptionArray(tmpifDescr,&tmpifDescr_size);
  tmpifDescr=NULL;
  tmpifDescr_size=0;
#ifdef AGNENT_DEBUG
  print_interfaceDescription(&in);
#endif

  result->__size=0;
  result->__offset=0;
  result->__ptr=NULL;
  res=0;
  /* return resulting interface description */
  res=setInterface(in,errmsg);
  if (res==1){
    update_InterfaceList();
    for(i=0;i<ifDescr_size;i++){
      if (strcmp(ifDescr[i].name,in.name)==0){
	++tmpifDescr_size; 
	tmpifDescr=(struct nsif__interfaceDescription *) realloc(tmpifDescr,tmpifDescr_size*sizeof(struct nsif__interfaceDescription));
	bzero(&tmpifDescr[tmpifDescr_size-1],sizeof(struct nsif__interfaceDescription));
	if (tmpifDescr==NULL) error("nsif__getInterfaceList","Not enough Memory",-2);
	cpifDescr(&tmpifDescr[tmpifDescr_size-1],&ifDescr[i]);
      }
    }   
    result->__size=tmpifDescr_size;
    result->__offset=0;
    result->__ptr=tmpifDescr;
    return SOAP_OK;
  }
  else{
    return soap_receiver_fault(soap, "Handling Error occurred",errmsg); 
  }
  return SOAP_FAULT;
}





/* function called by soap_serve on a incoming
   getInterfaceStats-Request */
int nsif__getInterfaceStats(struct soap *soap,xsd__string ifname,struct ifstatlist  *result){

  getIfStats(ifname);
  result->__size=ifStat_size;
  result->__offset=0;
  result->__ptr=ifStat;
  return SOAP_OK;
}

/* frees the with parameter ri given routingInformation-array. The
   number of elements of this array will be determined by the
   parameter size.  */
struct nsrt__routingInformation * freeRoutingInformationArray(struct nsrt__routingInformation *ri,int *size){
  int i=0;
  if (ri!=NULL){
    for(i=0;i<*size;i++)
      freeRoutingInformation(&ri[i]);
    ri=NULL;
    size=0;
  }
  size=0;
  return ri;
}



/* copies an routingInformation struct from Parameter from to
   Parameter to. The to struct must not be a NULL-pionter */
void cprtInf(struct nsrt__routingInformation *to,struct nsrt__routingInformation *from){

  to->iface=calloc(strlen(from->iface)+1,sizeof(char));
  strcpy(to->iface,from->iface);
  to->mtu=from->mtu;
  to->metric=from->metric;
  to->refcnt=from->refcnt;
  to->use=from->use;
  to->window=from->window;
  to->irtt=from->irtt;
  to->__sizeflags=from->__sizeflags;
  
  /* Addresses */
 
  to->dst_addr=cp_ns__addr(to->dst_addr,from->dst_addr);
  to->gw_addr=cp_ns__addr(to->gw_addr,from->gw_addr);
  to->nm_addr=cp_ns__addr(to->nm_addr,from->nm_addr);

 
  /* Flags */
  if (from->__sizeflags>0){
    int flags=0;
    struct flagContainer rfc;
    flags=setRouteFlags(from);
    getRouteFlags(flags,&rfc);
    to->flag=rfc.array;
  }else{
    to->flag=NULL;
  }
} 



/* function called by soap_serve on a incoming
   getRoutingEntrys-Request */
int nsrt__getRoutingEntrys(struct soap *soap, struct nsrt__routingInformation in,struct routelist *result){
  int i=0;
  
  tmprtInf=freeRoutingInformationArray(tmprtInf,&tmprtInf_size);
  tmprtInf_size=0;
  
  
  if ((in.iface==NULL) || (strcmp(in.iface,"")==0)) {
    /* all routinformation are requested */
    result->__size=rtInf_size;
    result->__offset=0;
    result->__ptr=rtInf;
    return SOAP_OK;
  }else{
    
    for(i=0;i<rtInf_size;i++){
      if (strcmp(rtInf[i].iface,in.iface)==0){
	++tmprtInf_size;
	tmprtInf=(struct nsrt__routingInformation *) realloc(tmprtInf,tmprtInf_size*sizeof(struct nsrt__routingInformation));
	bzero(&tmprtInf[tmprtInf_size-1],sizeof(struct nsrt__routingInformation));
	if (tmprtInf==NULL) error("nsrt__getRoutingTable","Not enough Memory",-2);
	cprtInf(&tmprtInf[tmprtInf_size-1],&rtInf[i]);
      }
    }
    result->__size=tmprtInf_size;
    result->__offset=0;
    result->__ptr=tmprtInf;
    return SOAP_OK;
  }
  return SOAP_FAULT;
} 

/* function called by soap_serve on a incoming
   registerEventHandler-Request */
int nsevt__registerEventHandler(struct soap *soap,xsd__int portnumber,xsd__string hostname,enum nsevt__regAction  regact,enum nsevt__regType tp, xsd__int *result){
  int res=0;
  int id=-1;
  

  
  if (soap->header){       // a Header was received
    id= soap->header->connection_ID; 
#ifdef AGENT_DEBUG
    printf("HEADER WITH ID=%d received\n",id);
#endif
  }

  printf("nsevt__REGISTEREVENTHANDLER for Host: %s, Port %d , ID=%d\n",hostname,portnumber,id);
  switch (regact){ 
  case EVT_REGISTER: res=register_eventManager(&id,hostname,portnumber,tp);break;
  case EVT_UNREGISTER: res=unregister_eventManager(&id,hostname,portnumber,tp);break;
  }

  /* evaluate the result of the requested operation */
  switch (res){
  case EM_NOTREGISTERED: *result=EVT_TP_FAULT;
    return soap_receiver_fault(soap, "Register exception occurred","Operation not possible, the receiver is not registered");
    break;
  case EM_SUCCESS      : *result=EVT_TP_SUCCESS;break; 
  case EM_WRONGHOSTNAME: *result=EVT_TP_FAULT;
    return soap_receiver_fault(soap, "Register exception occurred","The tranmitted Hostname is not valid");
    break;
  case EM_WRONGREGTYPE: *result=EVT_TP_FAULT;
    return soap_receiver_fault(soap, "Register exception occurred","The transmitted registrationtype is not supported by this agent");
    break;
  case EM_WRONGID: *result=EVT_TP_FAULT;
    return soap_receiver_fault(soap, "Register exception occurred","The transmitted ID is not valid");
    break;
  case EM_FLAGNOTSET: *result=EVT_TP_FAULT;
    return soap_receiver_fault(soap, "Register exception occurred","Cannot unregister type, type not set");  
    break;
  }

  if (!soap->header) {
    soap->header = soap_malloc(soap,sizeof(struct SOAP_ENV__Header)); // alloc new header
    soap->header->connection_ID = id; 
  }else{
    soap->header->connection_ID = id; 
  }


  return SOAP_OK;
}




/** NOT IMPLEMENTED, IT HAS TO BE INSTANTIATED DUE TO ITS REFERENCE IN THE STUB-FILES -> sect 6.2.10 
    gsoap 1.2 documentation **/ 
int nsevt__signalEvent(struct soap *soap, char *msg, enum nsevt__type type,enum nsevt__type *ret){
  return SOAP_NO_METHOD; // dummy: will never be called 
}


/* prints usage information to stdout */
void help(){
  printf("usage: SOAPagent [-ph?]\n\n");
  printf("-p, --port NUMBER      listening portnumber of the agent [DEFAULT:10080]\n");
  printf("-h, --host HOSTNAME    litening hostname of the agent    [DEFAULT:localhost]\n");
  printf("-?, --help             this output\n");       

}

/* close the agent and notify all connected eventreceivers */
void close_agent(){
  printf("Closing connection,exiting agent...\n");
  sendEventMessage("AGENT SHUTTING DOWN",EVT_TP_SRVCLOSE);
  soap_done(&soap); // close master socket
  exit(0);
}


/* initialises all global datastructures and sets the signal
   handling */
void init(){
  rtInf=NULL;
  rtInf_size=0;
  ifDescr=NULL;
  ifDescr_size=0;
  ifStat=NULL;
  ifStat_size=0;
  tmprtInf=NULL;
  tmprtInf_size=0;
  tmpifDescr=NULL;
  tmpifDescr_size=0;
  tmpifStat=NULL;
  tmpifStat_size=0;

  regarr=NULL;
  regarr_size=0;
  lastid=123;

  updateServiceInformation();

  signal(SIGHUP,close_agent);
  signal(SIGINT,close_agent);
  signal(SIGKILL,close_agent); 
}



int main(int argc,char *argv[]){

  int m,s;   // master and slave sockets
  int i;
  int opt_idx;
  char c;
  char optstring[5]="p:h:?";
  char host[30]="localhost";
  int port=10080;
  struct in_addr ipaddr;


  static struct option long_options[] =
             {
               {"port", 1, 0, 'p'},
               {"host", 1, 0, 'h'},
               {"help", 0, 0, '?'},
               {0, 0, 0, 0}
             };



 
  
  while((c=getopt_long(argc,argv,optstring,long_options,&opt_idx))!=-1){
    switch (c) {
    case 'p': port=strtol(optarg,NULL,10);break;
    case 'h': strcpy(host,optarg);break;
    case '?': help();exit(1);break;
    default: printf("UNKNOWN=%c",c);printf ("Option %s", long_options[opt_idx].name);
      break;
    }
  }

  printf("\n");
  if (optind < argc)
           {
             printf ("Nichtoptionselemente von ARGV: ");
             while (optind < argc)
             printf ("%s ", argv[optind++]);
             printf ("\n");
	     help();exit(1);
	   }

  
   init();

  printf("Initialising Soap...\n");
  soap_init(&soap); /* initialize runtime environment */ 
  soap.accept_timeout = 5;     /* these values are set to make the communication bahvior nonblocking */
  soap.send_timeout = 3;
  soap.recv_timeout = 3;
  rm_soap=&soap;
  m=soap_bind(&soap,host,port,100);
  soap.actor=NULL;  /* urn:WS_mgmt_stdEvent */


  if (m<0) soap_print_fault(&soap,stderr);
  else{
    fprintf(stderr,"Socket connection successful: master socket=%d\nServing incoming requests \n",m);
    for (i=1;;i++){
      s=soap_accept(&soap);
      updateServiceInformation();
      if (s<0){	
	/* TIMEOUT OCCURED ? */
      	if (s!=SOAP_EOF) soap_print_fault(&soap,stderr);
	/* handle updates */
      }else{
	/* incoming request */
	ipaddr.s_addr=soap.ip;
	//fprintf(stderr,"%d: accepted connection from IP=%s socket=%d\n",i,inet_ntoa(ipaddr),s);
	//fprintf(stderr, "Version=%d\n",soap.version) ;
     
	soap_serve(&soap); //process RPC request
	//fprintf(stderr,"request served\n");
	soap_destroy(&soap); // clean up class instance
	soap_end(&soap);     // clean up everything and close socket
      }
    }
      
    soap_done(&soap); // close master socket
  }
  return 0;
}
