#ifndef _AGENT_H
#define _AGENT_H

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <getopt.h>
#include <unistd.h>
#include <dnet.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "../headers/SOAPdevmgrStub.h"

#define  AGENT_IFNAMELEN 20
#define  AGENT_IPLEN     17
#define  AGENT_ETHLEN    18
#define  AGENT_FLAGLEN   16
#define  AGENT_ERRMSGLEN 250
#define  AGENT_BUFFLEN   250


struct time{
  int day;
  int hour;
  int min;
  int sec;
};
 
 
struct ut{
  struct time *uptime;
  struct time *idle;
};

struct flagContainer {
  int size;
  char **array;
};


struct nsif__interfaceDescription *ifDescr;
int ifDescr_size;

struct nsrt__routingInformation *rtInf;
int rtInf_size;

struct nsif__interfaceStats *ifStat;
int ifStat_size;

struct soap *rm_soap;              // second soap struct to invoke remothe methods soap

// interfaces.c

void create_InterfaceList();
void update_InterfaceList();
void getIfStats(char *ifname);
int setInterfaceFlags(struct nsif__interfaceDescription *id);
struct flagContainer * getInterfaceFlags(u_short flags,struct flagContainer *iffc);
void freeInterfaceDescription(struct nsif__interfaceDescription *id);
void print_interfaceDescription(struct nsif__interfaceDescription *ifd);
int setInterface(struct nsif__interfaceDescription in,char *errmsg);


#define _PROC_NET_DEV "/proc/net/dev"

// aux.c

void error(char *funk, char *err, int exitval);
void warning(char *funk, char *warn);
void print_ns__address(struct ns__address *ad);
struct ns__address * cp_ns__addr(struct ns__address *to,struct ns__address *from);
void freeAddrStruct(struct ns__address *addr);
// system.c
struct ut * readProc_uptime();
char * getSystemDescription();
char * getXSDDuration(struct time *tm);



// route.c

/* Keep this in sync with /usr/src/linux/include/linux/ipv6_route.h */
#ifndef RTF_DEFAULT
#define RTF_DEFAULT     0x00010000      /* default - learned via ND     */
#endif
#define RTF_ALLONLINK   0x00020000      /* fallback, no routers on link */
#ifndef RTF_ADDRCONF
#define RTF_ADDRCONF    0x00040000      /* addrconf route - RA          */
#endif

#define RTF_NONEXTHOP   0x00200000      /* route with no nexthop        */
#define RTF_EXPIRES     0x00400000
#define RTF_CACHE       0x01000000      /* cache entry                  */
#define RTF_FLOW        0x02000000      /* flow significant route       */
#define RTF_POLICY      0x04000000      /* policy route                 */
#define RTF_LOCAL       0x80000000
                                                                                            
/* Keep this in sync with /usr/src/linux/include/linux/route.h */
#define RTF_UP          0x0001          /* route usable                 */
#define RTF_GATEWAY     0x0002          /* destination is a gateway     */
#define RTF_HOST        0x0004          /* host entry (net otherwise)   */
#define RTF_REINSTATE   0x0008          /* reinstate route after tmout  */
#define RTF_DYNAMIC     0x0010          /* created dyn. (by redirect)   */
#define RTF_MODIFIED    0x0020          /* modified dyn. (by redirect)  */
#define RTF_MTU         0x0040          /* specific MTU for this route  */
#ifndef RTF_MSS
#define RTF_MSS         RTF_MTU         /* Compatibility :-(            */
#endif
#define RTF_WINDOW      0x0080          /* per route window clamping    */
#define RTF_IRTT        0x0100          /* Initial round trip time      */
#define RTF_REJECT      0x0200          /* Reject route                 */
                                                                                            
/* this is a 2.0.36 flag from /usr/src/linux/include/linux/route.h */
#define RTF_NOTCACHED   0x0400          /* this route isn't cached        */




#define _PROC_NET_ROUTE "/proc/net/route"

//#define _PROC_NET_ROUTE "../test/testroute.txt"


void create_routelist_by_Proc_net_route();
void update_RoutingInformation();
int checkFlagChange(int oldflag,int newflag,int flag,int *new,int *del);
void freeRoutingInformation(struct nsrt__routingInformation *ri);
int setRouteFlags(struct nsrt__routingInformation *ri);
struct flagContainer * getRouteFlags(int iflags,struct flagContainer *rfc);





// eventmanagemnt.c

struct notify{
  int id;                          // unique id for this connection (transported in the header-block) 
  unsigned long int hostip;        // in netbyteorder
  int port;
  int eventflags;                  // for further use, to set the eventype, a eventreceiver in interested in.
  unsigned short fail;                        // the number of unsuccessfull connection tries for this eventhandler; 
}; 

int lastid;


#define EM_MAX_TRIES 5       // the maximum number of unsuccesfull tries before the handler will be deleted            

#define EM_NOTREGISTERED  2
#define EM_SUCCESS        3
#define EM_WRONGHOSTNAME  4
#define EM_WRONGPARAMETER 5
#define EM_WRONGREGTYPE   6
#define EM_WRONGID        7
#define EM_FLAGNOTSET     8

#define EM_EVT_NONE      0x000
#define EM_EVT_ALL       0x001
#define EM_EVT_INTERFACE 0x002
#define EM_EVT_ROUTE     0x004







struct notify *regarr;
int regarr_size;

int register_eventManager(int *id,char* hostname,int port,enum nsevt__regType flag);
int unregister_eventManager(int *id,char* hostname,int port,enum nsevt__regType flag);
void sendEventMessage(char *msg,enum nsevt__type type);
#endif  /* _AGENT_H */
