
typedef char *xsd__string;
typedef double xsd__double;
typedef int xsd__int;
typedef char *xsd__address;
typedef short xsd__short;  
typedef char *xsd__duration;


enum ns__addrType {IP=1,ETH=2};

struct ns__address
{
  @ enum ns__addrType type;               /* Type of the address */
  xsd__string addr;                       /* Address encoded as String */ 
};


enum nsif__ifType {IF_NONE=0,IF_OTHER=1,IF_ETHERNET=6,
		   IF_LOOPBACK=24,IF_VIRTUAL=53}; /* other, 
						     Ethernet, 
						     software loopback, 
						     proprietary virtual/internal */



struct nsif__interfaceDescription 
{ 
  enum nsif__ifType   type;               /* Interface type     */
  xsd__string         name;               /* Interface name     */
  xsd__int            mtu;                /* Interface MTU      */
  int                 __sizeflags;        /* Flagarray Size     */     
  xsd__string         *flag;              /* Interface flagarray */
  struct ns__address  *address;           /* Interface address  */
  struct ns__address  *netmask;           /* Interface Netmask  */
  struct ns__address  *broadcast;         /* Broadcast Address  */
  struct ns__address  *dst_address;       /* point-to-point dst */
  struct ns__address  *link_address;      /* link-layer address */
  int                 __sizealias;        /* Aliasarray size    */     
  struct ns__address  **alias;            /* Interface aliasarray */

};


struct interfacelist
{ 
  struct nsif__interfaceDescription *__ptr;
  int __size;
  int __offset;
};



struct nsif__interfaceStats
{
  xsd__string     ifname;              /* Interface name     */  
  xsd__int        rx_bytes      0:1;   /* Received Bytes     */
  xsd__int        rx_packets    0:1;   /* Received Packets   */
  xsd__int        rx_errors     0:1;   /* Number of errors on ingoing traffic  */
  xsd__int        rx_drop       0:1;   /* Number of dropped packets */
  xsd__int        rx_fifo       0:1;   /* Number of ingoing FIFO buffer errors */
  xsd__int        rx_frame      0:1;   /* Number of ingoing packet framing errors.*/
  xsd__int        rx_compressed 0:1;   /* Number of compressed packets received */
  xsd__int        rx_multicast  0:1;   /* Number of received multicast frames */
  xsd__int        tx_bytes      0:1;   /* Transmitted Bytes     */
  xsd__int        tx_packets    0:1;   /* Transmitted Packets   */
  xsd__int        tx_errors     0:1;   /* Number of errors on outgoing traffic  */
  xsd__int        tx_drop       0:1;   /* Number of dropped packets */
  xsd__int        tx_fifo       0:1;   /* Number of outgoing FIFO buffer errors */
  xsd__int        tx_colls      0:1;   /* Number of collisions detected on the interface */ 
  xsd__int        tx_carrier    0:1;   /* Number of carrier losses detected by the device driver */
  xsd__int        tx_compressed 0:1;   /* Number of compressed packets transmitted */
};


struct ifstatlist
{ 
  struct nsif__interfaceStats *__ptr;
  int __size;
  int __offset;
};



struct nsrt__routingInformation 
{
  xsd__string         iface;                  /* Interface Name */
  struct ns__address *dst_addr;               /* Destination address */
  struct ns__address *gw_addr;                /* Gateway address */
  struct ns__address *nm_addr;                /* Netmask */
  int                __sizeflags;             /* Flagarray Size */     
  xsd__string        *flag;                   /* Interface flagarray  */
  xsd__int           mtu;                     /* Route maximum transfer unit */   
  xsd__int           metric;                  /* Route metric */ 
  xsd__int           refcnt;                  /* Number of references to this route. */
  xsd__int           use;                     /* Count  of lookups for the route. */
  xsd__int           window;                  /* TCP window size */
  xsd__int           irtt;                    /* initial round trip time */
}; 

 
/* FLAGS for nsrt__routingInformation 
   UP         (route is up)
   HOST       (target is a host)
   GATEWAY    (use gateway)
   REINSTATE  (reinstate route for dynamic routing)
   DYNAMIC    (dynamically installed by daemon or redirect)
   MODIFIED   (modified from routing daemon or redirect)
   ADDRCONF   (installed by addrconf)
   CACHE      (cache entry)
   REJECT     (reject route)(!)
*/


struct routelist
{ 
  struct nsrt__routingInformation *__ptr;
  int __size;
  int __offset;
};



struct nssys__sysInf{
  xsd__string     description 1:1;
  xsd__duration   uptime      0:1;
  xsd__duration   idletime    0:1;
};



int nssys__uptime(xsd__string *result);
int nssys__getSystemInformation(struct nssys__sysInf *result);
int nsif__getInterfaceInformation(struct nsif__interfaceDescription in,struct interfacelist  *result);
int nsif__setInterfaceInformation(struct nsif__interfaceDescription in,struct interfacelist  *result);
int nsif__getInterfaceStats(xsd__string ifname,struct ifstatlist  *result);
int nsrt__getRoutingEntrys(struct nsrt__routingInformation in,struct routelist *result);




struct SOAP_ENV__Header
{ xsd__int connection_ID;
};



enum nsevt__type {EVT_TP_SUCCESS=0,EVT_TP_IF=1,EVT_TP_ROUTE=2,EVT_TP_SRVCLOSE=3,EVT_TP_DEFAULT=4,EVT_TP_FAULT=5};
enum nsevt__regType{EVT_REGTYPE_NONE=0,EVT_REGTYPE_IF=2,EVT_REGTYPE_RT=3,EVT_REGTYPE_ALL=4,EVT_REGTYPE_FW=5};

int nsevt__registerEventHandler(xsd__int portnumber,xsd__string hostname,enum nsevt__regAction regact,enum nsevt__regType tp,xsd__int *result);

enum nsevt__regAction{EVT_REGISTER=1,EVT_UNREGISTER=2};

// importing 

  

int nsevt__signalEvent(xsd__string error,enum nsevt__type msgtype,enum nsevt__type *deliverySuccess);




// defining namespaces for the stub compiler

//gsoap nsevt service namespace: urn:WS_mgmt_stdEvent
//gsoap ns service namespace: urn:WS_mgmt_stdDefs
//gsoap nsif service namespace: urn:WS_mgmt_stdInterface
//gsoap nsrt service namespace: urn:WS_mgmt_stdRoute
//gsoap nssys service namespace: urn:WS_mgmt_stdSystem
