// Webservice Management 
// ERRORHANDLER
// Author: Oliver Bohle o.bohle@tu-bs.de
// Technische Universitt Braunschweig
#include "./Stub/headers/SOAPnsevtAgentStub.h" 
#include "./Stub/headers/myns.nsmap"

#include <string.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <signal.h>



struct soap soap;
char   srvhostname[50]="localhost";
int    srvport=10080;
char   myhostname[50]="localhost";
int    myport=10081;
int    progClose=0;
int    myID=-1;


void close_without_send();
void close_handler();

/* registers the eventreceiver at the SOAPagent */
int registerAtAgent(int *id,int srvport,char *srvhostname,int mp,char *mh,enum nsevt__regType regtp){
  char option[100]=""; 
  int fault=0;
  xsd__int resp;
  enum nsevt__regAction regact;
 
  bzero(option, sizeof(option));
  printf("trying to register ID=%d\n",*id);


  if ((soap.header==NULL) && (*id!=-1)) {
    printf("trying to register2 \n");
    soap.header = soap_malloc(&soap,sizeof(struct SOAP_ENV__Header)); // alloc new header
    soap.header->connection_ID = *id; 
  }else{
    if (soap.header!=NULL)
      soap.header->connection_ID = *id; 
  }  
  printf("trying to register 3\n");


  regact=EVT_REGISTER;
  sprintf(option,"%s:%d",srvhostname,srvport);
  printf("Trying to call nsevt_registerEventHandler with hostname:port %s:%d\n",srvhostname,srvport);
  fault=soap_call_nsevt__registerEventHandler(&soap,option,"",mp,mh,regact,regtp,&resp);
  printf("Errorcode: %d\n",fault);
  if (fault!=SOAP_OK) {
    soap_print_fault(&soap, stderr);
    close_without_send();
  }

  if (soap.header){ // a SOAP Header was received
    printf("TRANSACTION ID=%d\n",soap.header->connection_ID); 
    if (*id<0)
      *id=soap.header->connection_ID;
    else
      if (*id!=soap.header->connection_ID){
	printf("Received message with wrong ID! -> agent error.\n");
	exit(-1);
      }
  }

  if (resp!=EVT_TP_SUCCESS) {
    close_handler();
  }
  return fault;
}

/* unregisters the eventreceiver at the SOAPagent */
int unregisterAtAgent(int *id,int srvport,char *srvhostname,int mp,char *mh){
  char option[100]="";  
  int fault=0;
  xsd__int resp;
  //struct nsevt__registerErrorHandlerResponse resp;
  enum nsevt__regAction regact;
  enum nsevt__regType   regtp;

  bzero(option, sizeof(option));
  

  if ((!soap.header) &&(*id!=1)) {
    soap.header = soap_malloc(&soap,sizeof(struct SOAP_ENV__Header)); // alloc new header
    soap.header->connection_ID = *id; 
  }else{
    if (soap.header!=NULL)
      soap.header->connection_ID = *id; 
  } 

  regact=EVT_UNREGISTER;
  regtp=EVT_REGTYPE_ALL;
  sprintf(option,"%s:%d",srvhostname,srvport);
  printf("Trying to call nsevt_registerEventHandler with hostname:port %s:%d\n",srvhostname,srvport);
  fault=soap_call_nsevt__registerEventHandler(&soap,option,"",mp,mh,regact,regtp,&resp);
  printf("Errorcode: %d\n",fault);

  if (soap.header){ // a SOAP Header was received
    printf("TRANSACTION ID=%d\n",soap.header->connection_ID); 
    if (*id<0)
      *id=soap.header->connection_ID;
    else
      if (*id!=soap.header->connection_ID){
	printf("Received message with wrong ID! -> agent error.\n");
	exit(-1);
      }
  }


  if (fault!=SOAP_OK) soap_print_fault(&soap, stderr);
  // printf("ERGEBNIS:%d\n",resp);
  return fault;
}


/* called by soap_serve, the parameters are printed to stdout */
int nsevt__signalEvent(struct soap *soap, char *msg, enum nsevt__type type,enum nsevt__type *ret){
  *ret=EVT_TP_SUCCESS;
  if (soap->header) // a SOAP Header was received
    printf("TRANSACTION ID=%d\n",soap->header->connection_ID); 
  printf("EVENT OCCURED(%d): %s\n",type,msg);  
  if (type==EVT_TP_SRVCLOSE) progClose=1;  
  return SOAP_OK;
}


/** NOT IMPLEMENTED, IT HAS TO BE INSTANTIATED DUE TO ITS REFERENCE IN
    THE STUB-FILES -> sect 6.2.10 gsoap 1.2 documentation **/ 
int nsevt__registerEventHandler(struct soap *soap, int port, char *hostname, enum nsevt__regAction act, enum nsevt__regType tp,int *ret){
  return SOAP_NO_METHOD; // dummy: will never be called 
}


/* notifies the agent of the shutdown, closes the soap-socket and exits*/
void close_handler(){
  printf("Unregister at agent, closing connection, exiting eventreceiver...\n");
  unregisterAtAgent(&myID,srvport,srvhostname,myport,myhostname);
  soap_done(&soap); // close master socket
  exit(0);
}

/* closes the socket without contacting the agent and exits */
void close_without_send(){
  printf("Closing connection,exiting eventreceiver...\n");
  soap.accept_timeout = 1;     /* these values are set to make the communication bahvior nonblocking */
  soap.send_timeout = 1;
  soap.recv_timeout = 1;
  soap_done(&soap); // close master socket
  exit(0);
}

/* print usage information to stdout */
void help(){
  printf("usage: SOAPeventreceiver [-phrs?]\n\n");
  printf("-p, --port NUMBER       listening portnumber of the eventreceiver [DEFAULT:10081]\n");
  printf("-h, --host HOSTNAME     listening hostname of the eventreceiver    [DEFAULT:localhost]\n");
  printf("-r, --srvport NUMBER    listening portnumber of the agent         [DEFAULT:10080]\n");
  printf("-s, --srvhost HOSTNAME  listening hostname of the agent           [DEFAULT:localhost]\n");
  printf("-e  --evttype IF | RT   Request only events of a specific type\n");
  printf("                        IF = Interface , RT = Route \n");
  printf("-?, --help              this output\n");       

}


int main(int argc,char *argv[]){
  int m,s;   // master and slave sockets
  int opt_idx;
  char c;
  char optstring[12]="p:h:?r:s:e:f";
  int  infif=0,infrt=0,errreq=0;
  struct in_addr ipaddr;
 

  static struct option long_options[] =
             {
               {"port", 1, 0, 'p'},
               {"host", 1, 0, 'h'},
               {"srvport", 1, 0, 'r'},
	       {"srvhost", 1, 0, 's'},
	       {"evttype",1,0, 'e'},
	       {"help", 0, 0, '?'},
               {0, 0, 0, 0}
             };

  
  while((c=getopt_long(argc,argv,optstring,long_options,&opt_idx))!=-1){
    switch (c) {
    case 'p': myport=strtol(optarg,NULL,10);break;
    case 'h': strcpy(myhostname,optarg);break;
    case 'r': srvport=strtol(optarg,NULL,10);break;
    case 's': strcpy(srvhostname,optarg);break;
    case 'e': if (strcmp(optarg,"RT")==0) infrt=1;
              if (strcmp(optarg,"IF")==0) infif=1;
	      if (! (infif||infrt)) {printf("\n Wrong option \"%s\" in eventselection information\n\n",optarg);help();exit(1);}
              break;   
      
    case '?': help();exit(1);break;
    case 'f': errreq=1;break;
    default: printf("UNKNOWN=%c",c);printf ("Option %s", long_options[opt_idx].name);
      break;
    }
  }

  printf("\n");
  if (optind < argc)
           {
             printf ("Nichtoptionselemente von ARGV: ");
             while (optind < argc)
             printf ("%s ", argv[optind++]);
             printf ("\n");
	     help();exit(1);
	   }


  
  /* Install signal handler to ensure proper exit of the program and its network connections */
  signal(SIGHUP,close_handler);
  signal(SIGINT,close_handler);
  signal(SIGKILL,close_handler); 


  printf("Initialising Soap...\n");
  soap_init(&soap); // initialize runtime environment
  soap.send_timeout = 5; 
  soap.recv_timeout = 5;
  soap.actor=NULL;  /* urn:WS_mgmt_stdEvent */

 
 

  /* LISTENING CONSTRUCT */
  m=soap_bind(&soap,myhostname,myport,100);
  
  if (m<0) soap_print_fault(&soap,stderr);
  else{
     if (errreq==1) registerAtAgent(&myID,srvport,srvhostname,myport,myhostname,EVT_REGTYPE_FW);
     /*  register at the agent */ 
     if (!(infrt||infif))
       registerAtAgent(&myID,srvport,srvhostname,myport,myhostname,EVT_REGTYPE_ALL);
     else {
       if (infrt) registerAtAgent(&myID,srvport,srvhostname,myport,myhostname,EVT_REGTYPE_RT);
       if (infif) registerAtAgent(&myID,srvport,srvhostname,myport,myhostname,EVT_REGTYPE_IF);
     }



    fprintf(stderr,"Socket connection successful: master socket=%d\n",m);
    while(!progClose){
      s=soap_accept(&soap);     
      if (s<0){
	soap_print_fault(&soap,stderr);
	break;
      }
      ipaddr.s_addr=soap.ip;
      //fprintf(stderr,"%d: accepted connection from IP=%s socket=%d\n",i,inet_ntoa(ipaddr),s);
      //fprintf(stderr, "Version=%d\n",soap.version) ;
      
      soap_serve(&soap); //process RPC request
      //fprintf(stderr,"request served\n");
      soap_destroy(&soap); // clean up class instance
      soap_end(&soap);     // clean up everything and close socket
      
    }
    soap_done(&soap); // close master socket
  }
  close_without_send();
  return 0;
}
