// Webservice Management Application
// CLIENT
// Author: Oliver Bohle o.bohle@tu-bs.de
// Technische Universitt Braunschweig
#include "../headers/myns.nsmap"
#include "../headers/SOAPdevmgrStub.h"      
#include <string.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>



enum action {GET=1, SET =2};


char option[100];
struct soap soap;

char * getInterfaceType(enum ns__addrType at){
  if (at==IP) return "IP";
  if (at==ETH) return "ETH";
  return "NIXOS";
}



// ------------------ UPTIME -------------------------------
/* calls nssys__uptime and prints results to stdout */
void serve_HumanReadableUptime(){
  int fault;
   xsd__string uptimeresult[1];

  printf("Trying to call nssys_uptime with Option=%s\n",option);
  fault=soap_call_nssys__uptime(&soap,option,"",uptimeresult);
  printf("Errorcode: %d\n",fault);
  if (fault!=SOAP_OK) soap_print_fault(&soap, stderr);
  else 
    printf("UPTIME:%s\n",(*uptimeresult));
}

/* calls nssys__getSystemInformation and prints results to stdout */
void serve_SysStats(){
  struct nssys__sysInf si;
  int fault;
  printf("Trying to call nssys_getSystemInformation with Option=%s\n",option);
  fault=soap_call_nssys__getSystemInformation(&soap,option,"",&si);
  printf("Errorcode: %d\n",fault);
  if (fault!=SOAP_OK) soap_print_fault(&soap, stderr); 
  else {
    printf("DESCRIPTION : %s\n",si.description);
    printf("UPTIME      : %s\n",si.uptime);
    printf("IDLETIME    : %s\n",si.idletime);
  }
}









// ------------------ IFLIST ------------------------------
/* calls nsif__getInterfaceInformation or
   nsif__setInterfaceInformation determined by the action "act" and
   prints results to stdout. */
void serve_Interface(enum action act,struct nsif__interfaceDescription *ifdescr){
  int fault; 
  int i,j;
  struct interfacelist ifl;  
  
  if (act==GET) fault=soap_call_nsif__getInterfaceInformation(&soap,option,"",*ifdescr,&ifl);
  if (act==SET) fault=soap_call_nsif__setInterfaceInformation(&soap,option,"",*ifdescr,&ifl);
  //printf("Errorcode: %d\n",fault);
  if (fault!=SOAP_OK) soap_print_fault(&soap, stderr);
  else{
    
    if (ifl.__size>0){
      printf("INTERFACELIST:\n");
      for (i=0;i<ifl.__size;i++) {

	switch (ifl.__ptr[i].type){
	case IF_OTHER:   printf("other    = ");break;
	case IF_ETHERNET:printf("ETHERNET = ");break;
	case IF_LOOPBACK:printf("LOOPBACK = ");break;
	case IF_VIRTUAL: printf("VIRTUAL  = ");break;
	default:printf("NEW TYPE  =");break;
	}

	printf("%-10s: MTU %5d,",ifl.__ptr[i].name,ifl.__ptr[i].mtu);
	if (ifl.__ptr[i].address!=NULL) {
	  printf(" inet address=%s ",ifl.__ptr[i].address->addr);
	}
	if (ifl.__ptr[i].broadcast!=NULL)  printf(", broadcast=%s ",ifl.__ptr[i].broadcast->addr);
	if (ifl.__ptr[i].netmask!=NULL)  printf(", netmask=%s ",ifl.__ptr[i].netmask->addr);
	if (ifl.__ptr[i].dst_address!=NULL)  printf(", pointtopoint=%s ",ifl.__ptr[i].dst_address->addr);
	if (ifl.__ptr[i].link_address!=NULL)  printf(", hardware address=%s ",ifl.__ptr[i].link_address->addr);
	
	printf(",flags: ");
	for (j=0;j<ifl.__ptr[i].__sizeflags;j++){
	  printf(" %s",ifl.__ptr[i].flag[j]);
	  
	} 
	if (ifl.__ptr[i].__sizealias>0) printf(",alias: ");
	for (j=0;j<ifl.__ptr[i].__sizealias;j++){
	  printf(" %s,",ifl.__ptr[i].alias[j]->addr);
	
	} 
	printf("\n");
      }
    }else{
      printf("No interface with name \"%s\" found\n",ifdescr->name);
    }
  }
}




// ----------------------- IFSTAT ----------------------------
/* calls nsif__getInterfaceStats and prints results to stdout. */
void serve_IFstat(char *iface){
  int fault,i;
  struct ifstatlist ifstat;
 
  printf("serve_IFstat()\n");
  fault=soap_call_nsif__getInterfaceStats(&soap,option,"",iface,&ifstat);
  if (fault!=SOAP_OK) soap_print_fault(&soap, stderr);
  else{

    if (ifstat.__size>0){
      printf("INTERFACESTATS:\n");
      printf("          ||                            Receive Stats                                   ||                     Transmit Stats\n");
      printf("Name      || bytes    packets  errors   drop     fifo    frame     compressed multicast ||");
      printf(" bytes    packets  errors   drop     fifo     colls    carrier  compressed \n");
      printf("------------------------------------------------------------------------------------------");
      printf("---------------------------------------------------------------------------\n");
      for (i=0;i<ifstat.__size;i++) {
	printf("%-10s||",ifstat.__ptr[i].ifname);
	printf(" %-8d",ifstat.__ptr[i].rx_bytes);
	printf(" %-8d",ifstat.__ptr[i].rx_packets);
	printf(" %-8d",ifstat.__ptr[i].rx_errors);
	printf(" %-8d",ifstat.__ptr[i].rx_drop);
	printf(" %-8d",ifstat.__ptr[i].rx_fifo);
	printf(" %-8d",ifstat.__ptr[i].rx_frame);
	printf(" %-8d",ifstat.__ptr[i].rx_compressed);
	printf("   %-8d  ||",ifstat.__ptr[i].rx_multicast);
	printf(" %-8d",ifstat.__ptr[i].tx_bytes);
	printf(" %-8d",ifstat.__ptr[i].tx_packets);
	printf(" %-8d",ifstat.__ptr[i].tx_errors);
	printf(" %-8d",ifstat.__ptr[i].tx_drop);
	printf(" %-8d",ifstat.__ptr[i].tx_fifo);
	printf(" %-8d",ifstat.__ptr[i].tx_colls);
	printf(" %-8d",ifstat.__ptr[i].tx_carrier);
	printf(" %-8d",ifstat.__ptr[i].tx_compressed);
	printf("\n");
      }
    }else{
      printf("No interface with name \"%s\" found\n",iface);
    }
  }
}



// ----------------------- ROUTE ----------------------------
/* calls nsif__getRoutingEntrys and prints results to stdout. */
void serve_Route(struct nsrt__routingInformation *rtInf){
  int fault;
  int i,j;
  struct routelist rl;
    
  fault=soap_call_nsrt__getRoutingEntrys(&soap,option,"",*rtInf,&rl);
  //printf("Errorcode: %d\n",fault);
  if (fault!=SOAP_OK) soap_print_fault(&soap, stderr);
  else{
    if (rl.__size>0){
      printf("DESTINATION     GATWAY          NETMASK           METRIC REFCNT USE INTERFACE FLAGS\n");
      for (i=0;i<rl.__size;i++) {
	printf("%-15s",rl.__ptr[i].dst_addr->addr);
	printf("\t%-15s",rl.__ptr[i].gw_addr->addr);
	printf("\t%-15s",rl.__ptr[i].nm_addr->addr);
	printf("   %-3d",rl.__ptr[i].metric);
	printf("    %-3d",rl.__ptr[i].refcnt);
	printf("    %-3d",rl.__ptr[i].use);
	printf(" %-10s",rl.__ptr[i].iface);
	printf("<");
	for (j=0;j<rl.__ptr[i].__sizeflags;j++){
	  if (j!=0) printf(",");
	  printf("%s",rl.__ptr[i].flag[j]);
	}
	printf(">");	
	printf("\n");
      }
    }else{
      printf("No interface with name \"%s\" in routing table found\n",rtInf->iface);
    }

  }
}

/* prints usage information to sdtout */
void help(){
  printf("usage: SOAPmanager [-phrsia?] [SWITCH] \n\n");
  printf("-?, --help               this output\n");       
  printf("-p, --port NUMBER        listening portnumber of the agent [DEFAULT:10080]\n");
  printf("-h, --host HOSTNAME      listening hostname of the agent   [DEFAULT:localhost]\n");
  printf("-c, --cmd  GET/SET       specify command of the requsted operation [DEFAULT:GET]\n");
  printf("-s, --system             show agent information\n");
  printf("-u, --uptime             show uptime of the agent in human readable format\n");
  printf("-r, --route              show routing table\n");
  printf("-i, --interface OPTION   interface\n");
  printf("   OPTION=STAT           show interface statistics\n");
  printf("          INF            show interface information\n");
  printf("\n[SWITCH]\n");
  printf("    --ifname NAME        set the interface name\n"); 
  printf("    --dstaddr            set the destination address\n");
  printf("    --linkaddr           set the link address of the interface, only ethernet addresses are supported so far\n");
  printf("    --inetaddr           set the point-to-point address\n");
  printf("    --alias              set the aliases of an interface, a comma seperated list without whitespaces\n");
  printf("    --flag               set the flags of an interface, a comma seperated list without whitespaces\n");
}



int main(int argc,char *argv[]){
  int port=10080;
  char host[100]="localhost";
  char ifname[16]="",dstaddr[20],linkaddr[20],inetaddr[20],flags[256],alias[256];
  int srvrt=0,srvuptime=0,srvif=0,srvifstat=0,srvsysstat=0;
  int namopt=0,dstopt=0,linkopt=0,inetopt=0,flagopt=0,aliasopt=0;
  int optget=0,optset=0;
  int opt_idx;
  char c;
  char optstring[13]="p:h:?rsui:c:"; 
  
  static struct option long_options[] =
             {
               {"port",     1, 0, 'p'},
               {"host",     1, 0, 'h'},
               {"help",     0, 0, '?'},
	       {"cmd",      0, 0, 'c'},
	       {"route",    0, 0, 'r'},
	       {"interface",1, 0, 'i'},
	       {"system",   0, 0, 's'},
	       {"uptime",   0, 0, 'u'},
	       {"ifname",   1, 0, 0},
	       {"dstaddr",  1, 0, 0},
	       {"linkaddr", 1, 0, 0},
	       {"inetaddr", 1, 0, 0},
	       {"alias",    1, 0, 0},
	       {"flag",     1, 0, 0},
               {0, 0, 0, 0}
             };
  

 
 
soap_init(&soap); // initialize runtime environment

  while((c=getopt_long(argc,argv,optstring,long_options,&opt_idx))!=-1){
    switch (c) {
    case 'p': port=strtol(optarg,NULL,10);printf("PORT=%d \n",port);break;
    case 'h': strcpy(host,optarg);printf("HOST=%s \n",host);break;
    case 's': srvsysstat=1;break;
    case 'u': srvuptime=1;break;
    case 'i': if (strcmp(optarg,"STAT")==0) srvifstat=1;
              if (strcmp(optarg,"INF")==0) srvif=1;
	      if (! (srvif||srvifstat)) {printf("\n Wrong option \"%s\" in interface information Selection\n\n",optarg);help();exit(1);}
              break;
    case 'c': if (strcmp(optarg,"GET")==0) optget=1;
              if (strcmp(optarg,"SET")==0) optset=1;
	      if (! (optset||optget)) {printf("\n Wrong option \"%s\" in command selection\n\n",optarg);help();exit(1);}
              break;	      
    case 'n': 
    case 'r': srvrt=1;break;
    case 'a': printf("SHOW ALL\n");break;  
    case '?': help();exit(1);break;
    case 0: if (!strcmp(long_options[opt_idx].name,"ifname")) {namopt=1;strncpy(ifname,optarg,16);break;}
            if (!strcmp(long_options[opt_idx].name,"dstaddr")) {dstopt=1;strncpy(dstaddr,optarg,20);break;}
	    if (!strcmp(long_options[opt_idx].name,"linkaddr")) {linkopt=1;strncpy(linkaddr,optarg,20);break;}
	    if (!strcmp(long_options[opt_idx].name,"inetaddr")) {inetopt=1;strncpy(inetaddr,optarg,20);break;}
	    if (!strcmp(long_options[opt_idx].name,"flag")) {flagopt=1;strncpy(flags,optarg,256);break;}
	    if (!strcmp(long_options[opt_idx].name,"alias")) {aliasopt=1;strncpy(alias,optarg,256);break;}
	    break;
    default: printf("UNKNOWN=%c",c);printf ("Option %s", long_options[opt_idx].name);
      break;
    }
  } 
  


  if (optind < argc)
    {
      printf ("Nichtoptionselemente von ARGV: ");
      while (optind < argc)
	printf ("%s ", argv[optind++]);
      printf ("\n");
    }
  

  if (!(optset||optget)) optget=1;

  /**   REMOTE METHOD INVOCATION START **/
  sprintf(option,"%s:%d",host,port);
  printf("Initialising Soap... \n");
  
  if (srvif){
    struct nsif__interfaceDescription ifd;
    enum action act;
    char *tok=NULL;

    if (optget) act=GET;
    if (optset) act=SET;


    if (namopt){
      ifd.name=(char *)calloc(strlen(ifname)+1,sizeof(char));
      strcpy(ifd.name,ifname);
    } else ifd.name=NULL;
    
    if (inetopt){
      ifd.address=(struct ns__address *) malloc(sizeof(struct ns__address));
      ifd.address->type=IP;
      ifd.address->addr=(char *)calloc(strlen(inetaddr)+1,sizeof(char));
      strcpy(ifd.address->addr,inetaddr);     
    }else ifd.address=NULL;

    ifd.netmask=NULL;
    ifd.broadcast=NULL;

    if (dstopt){
      ifd.dst_address=(struct ns__address *) malloc(sizeof(struct ns__address));
      ifd.dst_address->type=IP;
      ifd.dst_address->addr=(char *)calloc(strlen(dstaddr)+1,sizeof(char));
      strcpy(ifd.dst_address->addr,dstaddr); 
    }else ifd.dst_address=NULL;

    if (linkopt){
      ifd.link_address=(struct ns__address *) malloc(sizeof(struct ns__address));
      ifd.link_address->type=ETH;
      ifd.link_address->addr=(char *)calloc(strlen(linkaddr)+1,sizeof(char));
      strcpy(ifd.link_address->addr,linkaddr); 
    }else ifd.link_address=NULL;


    if (aliasopt){
      ifd.__sizealias=0; 
      ifd.alias=NULL;
      tok=strtok(alias,",");
      while(tok != NULL){
	ifd.__sizealias++;
	ifd.alias=(struct ns__address **) realloc(ifd.alias,ifd.__sizealias*sizeof(struct ns__address *));
	ifd.alias[ifd.__sizealias-1]=(struct ns__address *) malloc (sizeof(struct ns__address));
	ifd.alias[ifd.__sizealias-1]->type=IP;
	ifd.alias[ifd.__sizealias-1]->addr=(char *)calloc(strlen(tok)+1,sizeof(char));
	strcpy(ifd.alias[ifd.__sizealias-1]->addr,tok); 
	tok=strtok(NULL,",");
      }
    }else{
      ifd.__sizealias=0;
      ifd.alias=NULL;
    }
    


    if (flagopt){
      ifd.__sizeflags=0;
      ifd.flag=NULL;
      tok=strtok(flags,",");
      while(tok != NULL){
	ifd.__sizeflags++;
	ifd.flag=(char **) realloc(ifd.flag,ifd.__sizeflags*sizeof(char *));
	ifd.flag[ifd.__sizeflags-1]=(char *) calloc (strlen(tok)+1,sizeof(char));
	strcpy(ifd.flag[ifd.__sizeflags-1],tok); 
	tok=strtok(NULL,",");
      }
    }else{
      ifd.__sizeflags=0;
      ifd.flag=0;
    }

    serve_Interface(act,&ifd);
  }
  
  /** INTERFACE STATS **/
  if (srvifstat) serve_IFstat(ifname);

  /** ROUTE INFORMATION **/
  if (srvrt){
    struct nsrt__routingInformation rti;

    if (namopt){
      rti.iface=(char *)calloc(strlen(ifname),sizeof(char));  
      strcpy(rti.iface,ifname);
    }else 
      rti.iface=NULL;

    rti.dst_addr=NULL;
    rti.gw_addr=NULL;
    rti.nm_addr=NULL;
    rti.__sizeflags=0;
    rti.flag=NULL;
    
    serve_Route(&rti);
  }
  if (srvsysstat) serve_SysStats();
  if (srvuptime) serve_HumanReadableUptime();

  soap_done(&soap); // close master socket
  return 0;
}

